@@ -120,7 +120,7 @@ About.prototype.initAboutApp = function(textBrowser) {
             + "<p>" + qsTr("%1 is an application for computer-aided design (CAD).").arg(this.applicationName) + "</p>"
             + "<p/>";
 
-            if (this.applicationName!=="QCAD" && !this.applicationName.startsWith("QCAD ")) {
+            if (this.applicationName!=="QCAD") {
                 html += "<p>" + qsTr("%1 is based on QCAD, a free (open source) software.").arg(this.applicationName) + "</p>";
             }
             else {
@@ -267,7 +267,8 @@ About.prototype.initAboutPlugins = function(textBrowser) {
             // plugin error:
             text = pluginInfo.get("Error");
             if (!isNull(text)) {
-                html += this.getTableRow(qsTr("Error:"), text);
+                html += this.getTableRow(qsTr("Error:"),
+                    "<span style='color:red'>" + text + "</span>", false);
             }
             html += "</table>";
             html += "<hr/>";
