@@ -240,6 +240,26 @@ function isBlock(obj) {
     return isOfType(obj, RBlock) || isOfType(obj, RBlockPointer);
 }
 
+/**
+ * Checks if the given object is a layout (RLayout).
+ *
+ * \return true if the given object is a layout.
+ */
+function isLayout(obj) {
+    return isOfType(obj, RLayout) || isOfType(obj, RLayoutPointer);
+}
+
+/**
+ * Checks if the given object is an RObject.
+ *
+ * \return true if the given object is an object (derived from RObject).
+ */
+function isObject(obj) {
+    return !isNull(obj) &&
+        isFunction(obj.getType) &&
+        isNumber(obj.getType());
+}
+
 /**
  * Checks if the given object is a graphics entity.
  *
@@ -248,6 +268,7 @@ function isBlock(obj) {
 function isEntity(obj) {
     return !isNull(obj) &&
         isFunction(obj.getType) && 
+        isFunction(obj.getData) &&
         isNumber(obj.getType());
 }
 
@@ -527,6 +548,15 @@ function isTextEntity(obj) {
     return isOfType(obj, RTextEntity) || isOfType(obj, RTextEntityPointer);
 }
 
+/**
+ * Checks if the given object is a simple text entity.
+ *
+ * \return true if the given object is a text entity (RTextEntity) and simple.
+ */
+function isSimpleTextEntity(obj) {
+    return isTextEntity(obj) && obj.isSimple();
+}
+
 /**
  * Checks if the given object is a block reference entity.
  *
@@ -2079,6 +2109,13 @@ function addDirtyFlag(title) {
 }
 
 function hasPlugin(name) {
+    if (name==="PROTOOLS") {
+        if (new QFileInfo("scripts/Pro").exists()) {
+            // PROTOOLS available not as plugins but as scripts folder:
+            return true;
+        }
+    }
+
     return RPluginLoader.hasPlugin(name);
 }
 
@@ -2139,8 +2176,76 @@ function restoreOverrideCursor() {
     }
 }
 
+function initUserShortcuts() {
+    var settings = RSettings.getQSettings();
+    var keys = RSettings.getAllKeys("Shortcuts");
+
+    var i, key, action;
+
+    for (i=0; i<keys.length; i++) {
+        key = keys[i];
+        action = RGuiAction.getByScriptFile(key);
+        if (isNull(action)) {
+            continue;
+        }
+        var scStringList = settings.value("Shortcuts/" + key);
+
+        // explicitely no shortcuts:
+        if (isNull(scStringList)) {
+            action.setShortcuts([]);
+            continue;
+        }
+
+        var scList = [];
+        for (var k=0; k<scStringList.length; k++) {
+            var sc = new QKeySequence(scStringList[k]);
+            scList.push(sc);
+        }
+        action.setShortcuts(scList);
+    }
+
+    keys = RSettings.getAllKeys("Commands");
+
+    for (i=0; i<keys.length; i++) {
+        key = keys[i];
+        action = RGuiAction.getByScriptFile(key);
+        if (isNull(action)) {
+            continue;
+        }
+        var cmStringList = settings.value("Commands/" + key);
+
+        // explicitely no commands:
+        if (isNull(cmStringList)) {
+            action.setCommands([]);
+            continue;
+        }
+
+        action.setCommands(cmStringList);
+    }
+}
+
 function addActionsToWidgets() {
+    initUserShortcuts();
+
     var appWin = RMainWindowQt.getMainWindow();
+
+    // delete previous shortcut delegates:
+    // part of workaround for QTBUG-38256, QTBUG-57990:
+    if ((RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) ||
+        (RSettings.getQtVersion()===0x050800 && RS.getSystemId()==="linux")) {
+
+        var scObjs = appWin.property("DelegatedShortcutsObjs");
+        if (!isNull(scObjs)) {
+            for (var si=0; si<scObjs.length; si++) {
+                if (isNull(scObjs[si])) {
+                    continue;
+                }
+                scObjs[si].destroy();
+            }
+        }
+        appWin.setProperty("DelegatedShortcutsObjs", []);
+    }
+
     var actions = RGuiAction.getActions();
     var widgetTypes = ["Menu", "ToolBar", "MatrixPanel", "Panel"];
     for (var c=0; c<actions.length; ++c) {
@@ -2191,25 +2296,30 @@ function addActionsToWidgets() {
                 if ((RSettings.getQtVersion()<0x050500 && RSettings.getQtVersion()>=0x050000) ||
                     (RSettings.getQtVersion()===0x050800 && RS.getSystemId()==="linux")) {
 
-                    if (isOfType(w.parentWidget(), QMenu) || RSettings.getQtVersion()===0x050800) {
-                        //new QShortcut(a.shortcut, a.parentWidget(), 0, 0,  Qt.WindowShortcut).activated.connect(a, "trigger");
-
+                    if (k===0) {
                         var shortcuts = a.shortcuts();
-                        for (var si=0; si<shortcuts.length; si++) {
-                            new QShortcut(shortcuts[si], a.parentWidget(), 0, 0,  Qt.WindowShortcut).activated.connect(a, "trigger");
-                        }
 
-                        // avoid 'Ambiguous shortcut overload' when tool buttons visible:
+                        var scObjs = appWin.property("DelegatedShortcutsObjs");
+                        if (isNull(scObjs)) {
+                            scObjs=[];
+                        }
+                        for (var si=0; si<shortcuts.length; si++) {
+                            var scObj = new QShortcut(shortcuts[si], a.parentWidget(), 0, 0, Qt.WindowShortcut)
+                            scObj.activated.connect(a, "trigger");
+                            scObjs.push(scObj);
+                        }
+                        appWin.setProperty("DelegatedShortcutsObjs", scObjs);
+
                         var sc = a.shortcut.toString();
                         if (sc.length>0) {
                             if (RS.getSystemId()==="linux") {
                                 a.setShortcutText("[" + sc + "]");
-                                //a.setShortcutText("\t" + sc);
                             }
                             else {
                                 a.setShortcutText(" " + sc);
                             }
                         }
+                        a.setProperty("DelegatedShortcuts", shortcuts);
                         a.setDefaultShortcuts([]);
                     }
                 }
